import { Comment } from '../models/Comment';
import { DatabaseService } from '../../db/services/Database';
import { CommentFetcher } from './CommentFetcher';

/**
 * CommentProcessor handles the retrieval and processing of comments from the r/Drama API.
 * It manages API requests through CommentFetcher, including rate limiting and retries, and coordinates
 * with DatabaseService for checking the existence and persisting new comments.
 */
export class CommentProcessor {

    /**
     * Fetches comments from the r/Drama API across multiple pages, up to the specified maximum.
     * Iterates through pages starting from the first page until the maximum page limit is reached
     * or there are no more comments to fetch. Each page's fetch operation is timed for performance analysis.
     * 
     * @returns {Promise<Comment[]>} A promise that resolves to an array of comments fetched from the API.
     * @throws {Error} Throws an error if there is a failure in fetching comments from the API.
     */
    static async processComments(maxPages: number = 10): Promise<Comment[]> {
        let comments: Comment[] = [];
        let stopFetching = false;

        for (let page = 1; page <= maxPages && !stopFetching; page++) {
            const newComments = await CommentFetcher.fetchComments(page)

            // Check each new comment against the database and existing comments in this batch
            for (const comment of newComments) {
                // Check if the comment was already processed in this batch
                if (comments.some(c => c.id === comment.id)) continue;

                const exists = await DatabaseService.commentExists(comment.id.toString());
                if (exists) {
                    stopFetching = true;
                    break; // Stop processing this batch of comments
                }
                await DatabaseService.insertComment(comment)
                comments.push(comment);
            }

            if (newComments.length === 0) break; // No more comments to fetch

        }
        return comments;
    }
}
