import fs from 'fs';
import path from 'path';

/**
 * Manages the retrieval and formatting of messages stored in text files.
 * This class provides functionality to load messages for rDrama and Reddit,
 * select a random message, and replace placeholders within that message
 * with specified values.
 */
export class MessageService {

    /**
     * Loads rDrama messages from a text file, splitting by a specific delimiter.
     * Each message is separated by '---END---' in the text file.
     * 
     * @example
     * const rdramaMessages = MessageService.loadRdramaMessages();
     * 
     * @returns {string[] | undefined} An array of rDrama messages, or undefined if there was an error loading the messages.
     */
    private static loadRdramaMessages(): string[] | undefined {
        try {
            const rdramaMessagesPath = path.join(__dirname, '..', 'messages', 'rdrama_messages.txt');
            return fs.readFileSync(rdramaMessagesPath, 'utf-8').split('---END---').filter(line => line.trim());
        } catch (error) {
            console.error('Failed to load rDrama messages:', error);
        }
    }

    /**
     * Loads Reddit messages from a text file, splitting by a specific delimiter.
     * Each message is separated by '---END---' in the text file.
     * 
     * @example
     * const redditMessages = MessageService.loadRedditMessages();
     * 
     * @returns {string[] | undefined} An array of Reddit messages, or undefined if there was an error loading the messages.
     */
    private static loadRedditMessages(): string[] | undefined {
        try {
            const redditMessagesPath = path.join(__dirname, '..', 'messages', 'reddit_messages.txt');
            return fs.readFileSync(redditMessagesPath, 'utf-8').split('---END---').filter(line => line.trim());
        } catch (error) {
            console.error('Failed to load Reddit messages:', error);
        }
    }

    /**
     * Selects a random Reddit message from the loaded messages and replaces placeholders within it.
     * 
     * @example
     * const message = MessageService.getRandomRedditMessage({ username: 'exampleUser' });
     * 
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string | undefined} A formatted Reddit message with placeholders replaced, or undefined if messages couldn't be loaded.
     */
    public static getRandomRedditMessage(placeholders: { [key: string]: string }): string | undefined {
        const redditMessages = this.loadRedditMessages()
        if (!redditMessages) return
        const message = redditMessages[Math.floor(Math.random() * redditMessages.length)];
        return this.replacePlaceholders(message, placeholders);
    }

    /**
     * Selects a random rDrama message from the loaded messages and replaces placeholders within it.
     * 
     * @example
     * const message = MessageService.getRandomRdramaMessage({ username: 'exampleUser' });
     * 
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string | undefined} A formatted rDrama message with placeholders replaced, or undefined if messages couldn't be loaded.
     */
    public static getRandomRdramaMessage(placeholders: { [key: string]: string }): string | undefined {
        const rdramaMessages = this.loadRdramaMessages()
        if (!rdramaMessages) return
        const message = rdramaMessages[Math.floor(Math.random() * rdramaMessages.length)];
        return this.replacePlaceholders(message, placeholders);
    }

    /**
     * Replaces placeholders in a message with values from a provided mapping.
     * 
     * @example
     * const formattedMessage = MessageService.replacePlaceholders('Hello, {username}!', { username: 'exampleUser' });
     * 
     * @param {string} message - The message containing placeholders.
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string} The message with placeholders replaced by actual values.
     */
    private static replacePlaceholders(message: string, placeholders: { [key: string]: string }): string {
        return Object.keys(placeholders).reduce((acc, key) => {
            const regex = new RegExp(`{${key}}`, 'g');
            return acc.replace(regex, placeholders[key]);
        }, message);
    }
}